@@ -284,7 +284,6 @@ input_device::input_device(input_manager &manager, std::string_view name, std::s
 	, m_internal(internal)
 	, m_threshold(std::max<s32>(s32(manager.machine().options().joystick_threshold() * osd::input_device::ABSOLUTE_MAX), 1))
 	, m_steadykey_enabled(manager.machine().options().steadykey())
-	, m_lightgun_reload_button(manager.machine().options().offscreen_reload())
 {
 }
 
@@ -764,24 +763,8 @@ input_device_switch_item::input_device_switch_item(
 
 s32 input_device_switch_item::read_as_switch(input_item_modifier modifier)
 {
-	// if we're doing a lightgun reload hack, button 1 and 2 operate differently
-	input_device_class devclass = m_device.devclass();
-	if (devclass == DEVICE_CLASS_LIGHTGUN && m_device.lightgun_reload_button())
-	{
-		// button 1 is pressed if either button 1 or 2 are active
-		if (m_itemid == ITEM_ID_BUTTON1)
-		{
-			input_device_item *button2_item = m_device.item(ITEM_ID_BUTTON2);
-			if (button2_item != nullptr)
-				return button2_item->update_value() | update_value();
-		}
-
-		// button 2 is never officially pressed
-		if (m_itemid == ITEM_ID_BUTTON2)
-			return 0;
-	}
-
 	// steadykey for keyboards
+	input_device_class devclass = m_device.devclass();
 	if (devclass == DEVICE_CLASS_KEYBOARD && m_device.steadykey_enabled())
 		return m_steadykey;
 
@@ -1024,15 +1007,6 @@ s32 input_device_absolute_item::read_as_absolute(input_item_modifier modifier)
 	s32 result = m_device.adjust_absolute(update_value());
 	assert(result >= osd::input_device::ABSOLUTE_MIN && result <= osd::input_device::ABSOLUTE_MAX);
 
-	// if we're doing a lightgun reload hack, override the value
-	if (m_device.devclass() == DEVICE_CLASS_LIGHTGUN && m_device.lightgun_reload_button())
-	{
-		// if it is pressed, return (min,max)
-		input_device_item *button2_item = m_device.item(ITEM_ID_BUTTON2);
-		if (button2_item != nullptr && button2_item->update_value())
-			result = (m_itemid == ITEM_ID_XAXIS) ? osd::input_device::ABSOLUTE_MIN : osd::input_device::ABSOLUTE_MAX;
-	}
-
 	// positive/negative: scale to full axis
 	if (modifier == ITEM_MODIFIER_REVERSE)
 		result = -result;
