@@ -14,6 +14,7 @@
 #include "ui/inifile.h"
 #include "ui/selector.h"
 
+#include "infoxml.h"
 #include "language.h"
 #include "mame.h"
 
@@ -67,12 +68,15 @@ constexpr std::pair<char const *, char const *> SOFTWARE_INFO_NAMES[] = {
 		{ "distributor",        N_p("swlist-info", "Distributor")               },
 		{ "install",            N_p("swlist-info", "Installation Instructions") },
 		{ "isbn",               N_p("swlist-info", "ISBN")                      },
+		{ "language",           N_p("swlist-info", "Language")                  },
 		{ "oem",                N_p("swlist-info", "OEM")                       },
 		{ "original_publisher", N_p("swlist-info", "Original Publisher")        },
 		{ "partno",             N_p("swlist-info", "Part Number")               },
 		{ "pcb",                N_p("swlist-info", "PCB")                       },
 		{ "programmer",         N_p("swlist-info", "Programmer")                },
 		{ "release",            N_p("swlist-info", "Release Date")              },
+		{ "required_os",        N_p("swlist-info", "Required Operating System") },
+		{ "required_ram",       N_p("swlist-info", "Required System RAM")       },
 		{ "serial",             N_p("swlist-info", "Serial Number")             },
 		{ "usage",              N_p("swlist-info", "Usage Instructions")        },
 		{ "version",            N_p("swlist-info", "Version")                   } };
@@ -96,6 +100,7 @@ constexpr char const *MACHINE_FILTER_NAMES[machine_filter::COUNT] = {
 		N_p("machine-filter", "Clones"),
 		N_p("machine-filter", "Manufacturer"),
 		N_p("machine-filter", "Year"),
+		N_p("machine-filter", "Source File"),
 		N_p("machine-filter", "Save Supported"),
 		N_p("machine-filter", "Save Unsupported"),
 		N_p("machine-filter", "CHD Required"),
@@ -202,23 +207,16 @@ public:
 
 	virtual void show_ui(mame_ui_manager &mui, render_container &container, std::function<void (Base &)> &&handler) override
 	{
-		if (m_choices.empty())
-		{
-			handler(*this);
-		}
-		else
-		{
-			menu::stack_push<menu_selector>(
-					mui, container,
-					_("Filter"), // TODO: get localised name of filter in here somehow
-					std::vector<std::string>(m_choices), // ouch, a vector copy!
-					m_selection,
-					[this, cb = std::move(handler)] (int selection)
-					{
-						m_selection = selection;
-						cb(*this);
-					});
-		}
+		menu::stack_push<menu_selector>(
+				mui, container,
+				_("Filter"), // TODO: get localised name of filter in here somehow
+				std::vector<std::string>(m_choices), // ouch, a vector copy!
+				m_selection,
+				[this, cb = std::move(handler)] (int selection)
+				{
+					m_selection = selection;
+					cb(*this);
+				});
 	}
 
 	virtual bool wants_adjuster() const override { return have_choices(); }
@@ -685,7 +683,7 @@ class working_machine_filter_impl : public simple_filter_impl_base<machine_filte
 public:
 	working_machine_filter_impl(machine_filter_data const &data, char const *value, util::core_file *file, unsigned indent) { }
 
-	virtual bool apply(ui_system_info const &system) const override { return !(system.driver->flags & machine_flags::NOT_WORKING); }
+	virtual bool apply(ui_system_info const &system) const override { return !(system.driver->type.emulation_flags() & device_t::flags::NOT_WORKING); }
 };
 
 
@@ -748,7 +746,7 @@ class save_machine_filter_impl : public simple_filter_impl_base<machine_filter,
 public:
 	save_machine_filter_impl(machine_filter_data const &data, char const *value, util::core_file *file, unsigned indent) { }
 
-	virtual bool apply(ui_system_info const &system) const override { return system.driver->flags & machine_flags::SUPPORTS_SAVE; }
+	virtual bool apply(ui_system_info const &system) const override { return !(system.driver->type.emulation_flags() & device_t::flags::SAVE_UNSUPPORTED); }
 };
 
 
@@ -800,6 +798,18 @@ public:
 };
 
 
+class source_file_machine_filter : public choice_filter_impl_base<machine_filter, machine_filter::SOURCE_FILE>
+{
+public:
+	source_file_machine_filter(machine_filter_data const &data, char const *value, util::core_file *file, unsigned indent)
+		: choice_filter_impl_base<machine_filter, machine_filter::SOURCE_FILE>(data.source_files(), value)
+	{
+	}
+
+	virtual bool apply(ui_system_info const &system) const override { return !have_choices() || (selection_valid() && (selection_text() == info_xml_creator::format_sourcefile(system.driver->type.source()))); }
+};
+
+
 
 //-------------------------------------------------
 //  complementary machine filters
@@ -1227,6 +1237,7 @@ public:
 		case FAVORITE:
 		case MANUFACTURER:
 		case YEAR:
+		case SOURCE_FILE:
 		case CUSTOM:
 		case COUNT:
 			break;
@@ -1236,7 +1247,17 @@ public:
 
 	static bool is_inclusion(type n)
 	{
-		return (CATEGORY == n) || (MANUFACTURER == n) || (YEAR == n);
+		switch (n)
+		{
+		case CATEGORY:
+		case MANUFACTURER:
+		case YEAR:
+		case SOURCE_FILE:
+			return true;
+
+		default:
+			return false;
+		}
 	}
 
 private:
@@ -1602,10 +1623,21 @@ void machine_filter_data::add_year(std::string const &year)
 		m_years.emplace(pos, year);
 }
 
+void machine_filter_data::add_source_file(std::string_view path)
+{
+	std::vector<std::string>::iterator const pos(std::lower_bound(m_source_files.begin(), m_source_files.end(), path));
+	if ((m_source_files.end() == pos) || (*pos != path))
+		m_source_files.emplace(pos, path);
+}
+
 void machine_filter_data::finalise()
 {
+	for (std::string &path : m_source_files)
+		path = info_xml_creator::format_sourcefile(path);
+
 	std::stable_sort(m_manufacturers.begin(), m_manufacturers.end());
 	std::stable_sort(m_years.begin(), m_years.end());
+	std::stable_sort(m_source_files.begin(), m_source_files.end());
 }
 
 std::string machine_filter_data::extract_manufacturer(std::string const &manufacturer)
@@ -1780,6 +1812,8 @@ machine_filter::ptr machine_filter::create(type n, machine_filter_data const &da
 		return std::make_unique<manufacturer_machine_filter>(data, value, file, indent);
 	case YEAR:
 		return std::make_unique<year_machine_filter>(data, value, file, indent);
+	case SOURCE_FILE:
+		return std::make_unique<source_file_machine_filter>(data, value, file, indent);
 	case SAVE:
 		return std::make_unique<save_machine_filter>(data, value, file, indent);
 	case NOSAVE:
