@@ -21,7 +21,7 @@
 #include "bus/nscsi/cd.h"
 #include "bus/nscsi/hd.h"
 #include "cpu/i386/i386.h"
-#include "cpu/tms32031/tms32031.h"
+#include "cpu/tms320c3x/tms320c3x.h"
 #include "machine/53c7xx.h"
 #include "machine/bacta_datalogger.h"
 #include "machine/mc146818.h"
@@ -80,21 +80,21 @@ public:
 	void rs_config_base(machine_config &config);
 
 protected:
-	virtual void machine_reset() override;
-	virtual void machine_start() override;
-	virtual void video_start() override;
+	virtual void machine_reset() override ATTR_COLD;
+	virtual void machine_start() override ATTR_COLD;
+	virtual void video_start() override ATTR_COLD;
 
 	uint8_t interrupt_ctrl_r(offs_t offset);
 	void interrupt_ctrl_w(offs_t offset, uint8_t data);
 	uint8_t nvram_r(offs_t offset);
 	void nvram_w(offs_t offset, uint8_t data);
 
-	void cpu_map_base(address_map &map);
-	void io_map(address_map &map);
-	void dsp_map_base(address_map &map);
+	void cpu_map_base(address_map &map) ATTR_COLD;
+	void io_map(address_map &map) ATTR_COLD;
+	void dsp_map_base(address_map &map) ATTR_COLD;
 
 	required_device<i486_device>     m_maincpu;
-	required_device<tms3203x_device> m_dsp;
+	required_device<tms320c3x_device> m_dsp;
 	required_device<z80scc_device>   m_duart;
 
 	void update_irq(uint32_t which, uint32_t state);
@@ -151,8 +151,8 @@ private:
 	void port2_w(uint32_t data);
 	void port3_w(uint32_t data);
 	void dpylist_w(uint32_t data);
-	uint32_t tms32031_control_r(offs_t offset);
-	void tms32031_control_w(offs_t offset, uint32_t data);
+	uint32_t tms320c31_control_r(offs_t offset);
+	void tms320c31_control_w(offs_t offset, uint32_t data);
 	void dsp_unk_w(uint32_t data);
 	void dsp_ctrl_w(uint32_t data);
 	void dsp_486_int_w(uint32_t data);
@@ -160,20 +160,20 @@ private:
 	void dsp_speedup_w(uint32_t data);
 	uint32_t ncr53c700_read(offs_t offset, uint32_t mem_mask = ~0);
 	void ncr53c700_write(offs_t offset, uint32_t data, uint32_t mem_mask = ~0);
-	DECLARE_WRITE_LINE_MEMBER(scsi_irq);
+	void scsi_irq(int state);
 
 	TIMER_CALLBACK_MEMBER(tms_timer1);
 	TIMER_CALLBACK_MEMBER(tms_tx_timer);
-	DECLARE_WRITE_LINE_MEMBER(vblank_irq);
-	DECLARE_WRITE_LINE_MEMBER(duart_irq);
+	void vblank_irq(int state);
+	void duart_irq(int state);
 
 	uint32_t screen_update(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect);
 	void upload_palette(uint32_t word1, uint32_t word2);
 	IRQ_CALLBACK_MEMBER(irq_callback);
 	nscsi_connector &add_rastersp_scsi_slot(machine_config &config, const char *tag, const char *default_slot);
 	static void ncr53c700_config(device_t *device);
-	void cpu_map(address_map &map);
-	void dsp_map(address_map &map);
+	void cpu_map(address_map &map) ATTR_COLD;
+	void dsp_map(address_map &map) ATTR_COLD;
 	uint8_t interrupt_status_r(offs_t offset);
 
 	std::unique_ptr<uint8_t[]>   m_nvram8;
@@ -186,7 +186,6 @@ private:
 	bitmap_ind16 m_update_bitmap;
 
 	uint8_t m_port2_data = 0;
-	uint32_t m_last_watchdog = 0;
 	int m_left_volume = 0;
 	int m_right_volume = 0;
 	uint8_t m_interrupt_mask = 0;
@@ -206,11 +205,11 @@ public:
 
 	void fbcrazy(machine_config &config);
 
-	DECLARE_READ_LINE_MEMBER(meter_pulse_r);
+	int meter_pulse_r();
 
 protected:
-	virtual void machine_reset() override;
-	virtual void machine_start() override;
+	virtual void machine_reset() override ATTR_COLD;
+	virtual void machine_start() override ATTR_COLD;
 
 private:
 	required_ioport m_io_track_x;
@@ -223,11 +222,11 @@ private:
 	void aux_port3_w(offs_t offset, uint8_t data);
 	void aux_port4_w(offs_t offset, uint8_t data);
 
-	void cpu_map(address_map &map);
-	void dsp_map(address_map &map);
+	void cpu_map(address_map &map) ATTR_COLD;
+	void dsp_map(address_map &map) ATTR_COLD;
 
 	TIMER_CALLBACK_MEMBER(trackball_timer);
-	DECLARE_WRITE_LINE_MEMBER(trackball_rts);
+	void trackball_rts(int state);
 
 	uint8_t m_aux_port3_data;
 	uint8_t m_trackball_ctr;
@@ -272,7 +271,6 @@ void rastersp_state::machine_start()
 	save_item(NAME(m_speedup_count));
 	save_item(NAME(m_tms_io_regs));
 	save_item(NAME(m_port2_data));
-	save_item(NAME(m_last_watchdog));
 	save_item(NAME(m_left_volume));
 	save_item(NAME(m_right_volume));
 	save_item(NAME(m_interrupt_mask));
@@ -299,7 +297,6 @@ void rastersp_state::machine_reset()
 	m_dlba = 0;
 	m_palette_number = 0;
 
-	m_last_watchdog = 0;
 	m_port2_data = 0;
 	m_left_volume = 0;
 	m_right_volume = 0;
@@ -312,8 +309,8 @@ void rastersp_state::machine_reset()
 	m_maincpu->set_input_line(INPUT_LINE_RESET, ASSERT_LINE);
 
 	// Set IRQ1 line to cause DSP to boot from 0x400000
-	m_dsp->set_input_line(TMS3203X_IRQ1, ASSERT_LINE);
-	m_dsp->set_input_line(TMS3203X_IRQ1, CLEAR_LINE);
+	m_dsp->set_input_line(TMS320C3X_IRQ1, ASSERT_LINE);
+	m_dsp->set_input_line(TMS320C3X_IRQ1, CLEAR_LINE);
 
 	// Reset DSP internal registers
 	m_tms_io_regs[SPORT_GLOBAL_CTL] = 0;
@@ -607,31 +604,31 @@ void rastersp_state::update_irq(uint32_t which, uint32_t state)
 }
 
 
-WRITE_LINE_MEMBER( rastersp_state::scsi_irq )
+void rastersp_state::scsi_irq(int state)
 {
 	update_irq(IRQ_SCSI, state);
 
 	if (state && BIT(m_dsp_ctrl_data, 5))
 	{
-		m_dsp->set_input_line(TMS3203X_IRQ0, ASSERT_LINE);
-		m_dsp->set_input_line(TMS3203X_IRQ0, CLEAR_LINE);
+		m_dsp->set_input_line(TMS320C3X_IRQ0, ASSERT_LINE);
+		m_dsp->set_input_line(TMS320C3X_IRQ0, CLEAR_LINE);
 	}
 }
 
 
-WRITE_LINE_MEMBER( rastersp_state::vblank_irq )
+void rastersp_state::vblank_irq(int state)
 {
 	if (state)
 		update_irq(IRQ_VBLANK, ASSERT_LINE);
 }
 
-WRITE_LINE_MEMBER( rastersp_state::duart_irq )
+void rastersp_state::duart_irq(int state)
 {
 	update_irq(IRQ_UART, state);
 }
 
 
-WRITE_LINE_MEMBER( fbcrazy_state::trackball_rts )
+void fbcrazy_state::trackball_rts(int state)
 {
 	m_trackball_enabled = state;
 }
@@ -759,17 +756,11 @@ void rastersp_state::port1_w(uint32_t data)
 	*/
 	if (BIT(data, 5) && BIT(m_dsp_ctrl_data, 5))
 	{
-		m_dsp->set_input_line(TMS3203X_IRQ2, ASSERT_LINE);
-		m_dsp->set_input_line(TMS3203X_IRQ2, CLEAR_LINE);
+		m_dsp->set_input_line(TMS320C3X_IRQ2, ASSERT_LINE);
+		m_dsp->set_input_line(TMS320C3X_IRQ2, CLEAR_LINE);
 	}
 
-	if (BIT(data, 7) != m_last_watchdog)
-	{
-		m_last_watchdog = BIT(data, 7);
-
-		if (BIT(data, 7))
-			m_watchdog->watchdog_reset();
-	}
+	m_watchdog->reset_line_w(BIT(data, 7));
 }
 
 
@@ -899,7 +890,7 @@ void fbcrazy_state::aux_port4_w(offs_t offset, uint8_t data)
 	// Bit 7 - 5p lockout
 }
 
-READ_LINE_MEMBER(fbcrazy_state::meter_pulse_r)
+int fbcrazy_state::meter_pulse_r()
 {
 	return m_aux_port3_data & 0xbc ? 1 : 0;
 }
@@ -924,7 +915,7 @@ void rastersp_state::interrupt_ctrl_w(offs_t offset, uint8_t data)
 uint8_t rastersp_state::interrupt_status_r(offs_t offset)
 {
 	// TODO: This returns something to do with interrupts
-	// Is it a bit pattern of the current interupt or a
+	// Is it a bit pattern of the current interrupt or a
 	// bit pattern of all interrupts pending?
 	// Either way, 0 works anyway!
 	return 0;
@@ -958,8 +949,8 @@ TIMER_CALLBACK_MEMBER(rastersp_state::tms_tx_timer)
 	// Signal a transmit interrupt
 	if (m_tms_io_regs[SPORT_GLOBAL_CTL] & (1 << 23))
 	{
-		m_dsp->set_input_line(TMS3203X_XINT0, ASSERT_LINE);
-		m_dsp->set_input_line(TMS3203X_XINT0, CLEAR_LINE);
+		m_dsp->set_input_line(TMS320C3X_XINT0, ASSERT_LINE);
+		m_dsp->set_input_line(TMS320C3X_XINT0, CLEAR_LINE);
 	}
 }
 
@@ -969,7 +960,7 @@ TIMER_CALLBACK_MEMBER(rastersp_state::tms_timer1)
 }
 
 
-uint32_t rastersp_state::tms32031_control_r(offs_t offset)
+uint32_t rastersp_state::tms320c31_control_r(offs_t offset)
 {
 	uint32_t val = m_tms_io_regs[offset];
 
@@ -984,7 +975,7 @@ uint32_t rastersp_state::tms32031_control_r(offs_t offset)
 		}
 		default:
 		{
-			logerror("TMS32031: Unhandled I/O read: %x\n", offset);
+			logerror("TMS320C31: Unhandled I/O read: %x\n", offset);
 		}
 	}
 
@@ -992,7 +983,7 @@ uint32_t rastersp_state::tms32031_control_r(offs_t offset)
 }
 
 
-void rastersp_state::tms32031_control_w(offs_t offset, uint32_t data)
+void rastersp_state::tms320c31_control_w(offs_t offset, uint32_t data)
 {
 	uint32_t old = m_tms_io_regs[offset];
 
@@ -1047,7 +1038,7 @@ void rastersp_state::tms32031_control_w(offs_t offset, uint32_t data)
 		}
 		default:
 		{
-			logerror("TMS32031: Unhandled I/O write: %x %x\n", offset, data);
+			logerror("TMS320C31: Unhandled I/O write: %x %x\n", offset, data);
 		}
 	}
 }
@@ -1145,7 +1136,8 @@ void rastersp_state::io_map(address_map &map)
 	map(0x100c, 0x100f).portr("DSW2");
 	map(0x1010, 0x1013).portr("DSW1");
 	map(0x1014, 0x1017).portr("EXTRA");
-	map(0x4000, 0x4007).rw("rtc", FUNC(mc146818_device::read), FUNC(mc146818_device::write)).umask32(0x000000ff);
+	map(0x4000, 0x4000).w("rtc", FUNC(mc146818_device::address_w));
+	map(0x4004, 0x4004).rw("rtc", FUNC(mc146818_device::data_r), FUNC(mc146818_device::data_w));
 	map(0x6000, 0x6000).rw( m_duart, FUNC(z80scc_device::cb_r), FUNC(z80scc_device::cb_w));
 	map(0x6004, 0x6004).rw( m_duart, FUNC(z80scc_device::db_r), FUNC(z80scc_device::db_w));
 	map(0x6008, 0x6008).rw( m_duart, FUNC(z80scc_device::ca_r), FUNC(z80scc_device::ca_w));
@@ -1155,7 +1147,7 @@ void rastersp_state::io_map(address_map &map)
 void rastersp_state::dsp_map_base(address_map &map)
 {
 	map(0x400000, 0x40ffff).rom().region("dspboot", 0);
-	map(0x808000, 0x80807f).rw(FUNC(rastersp_state::tms32031_control_r), FUNC(rastersp_state::tms32031_control_w));
+	map(0x808000, 0x80807f).rw(FUNC(rastersp_state::tms320c31_control_r), FUNC(rastersp_state::tms320c31_control_w));
 	map(0x880402, 0x880402).w(FUNC(rastersp_state::dsp_unk_w));
 	map(0x883c00, 0x883c00).w(FUNC(rastersp_state::dsp_486_int_w));
 	map(0xc00000, 0xc03fff).bankrw("bank2");
@@ -1406,7 +1398,7 @@ static INPUT_PORTS_START( fbcrazy )
 	PORT_DIPSETTING(    0x08, DEF_STR( No ) )
 	PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_UNUSED )
 	PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_UNUSED )
-	PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_CUSTOM ) PORT_READ_LINE_MEMBER(fbcrazy_state, meter_pulse_r)
+	PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_CUSTOM ) PORT_READ_LINE_MEMBER(FUNC(fbcrazy_state::meter_pulse_r))
 	PORT_BIT( 0x80, IP_ACTIVE_LOW, IPT_UNUSED )
 
 	PORT_START("TRACK_X")  // Fake trackball input port
@@ -1449,7 +1441,7 @@ void rastersp_state::rs_config_base(machine_config &config)
 	m_maincpu->set_irq_acknowledge_callback(FUNC(rastersp_state::irq_callback));
 	m_maincpu->set_addrmap(AS_IO, &rastersp_state::io_map);
 
-	TMS32031(config, m_dsp, 33'330'000);
+	TMS320C31(config, m_dsp, 33'330'000);
 	m_dsp->set_mcbl_mode(true); // Boot-loader mode
 
 	MC146818(config, "rtc", 32.768_kHz_XTAL);
@@ -1480,13 +1472,12 @@ void rastersp_state::rs_config_base(machine_config &config)
 	PALETTE(config, m_palette, palette_device::RGB_565);
 
 	/* Sound */
-	SPEAKER(config, "lspeaker").front_left();
-	SPEAKER(config, "rspeaker").front_right();
+	SPEAKER(config, "speaker", 2).front();
 
 	DAC_16BIT_R2R_TWOS_COMPLEMENT(config, m_ldac, 0);
 	DAC_16BIT_R2R_TWOS_COMPLEMENT(config, m_rdac, 0);
-	m_ldac->add_route(ALL_OUTPUTS, "lspeaker", 0.5); // unknown DAC
-	m_rdac->add_route(ALL_OUTPUTS, "rspeaker", 0.5); // unknown DAC
+	m_ldac->add_route(ALL_OUTPUTS, "speaker", 0.5, 0); // unknown DAC
+	m_rdac->add_route(ALL_OUTPUTS, "speaker", 0.5, 1); // unknown DAC
 
 	SCC85C30(config, m_duart, 8'000'000);
 	m_duart->configure_channels(1'843'200, 0, 1'843'200, 0);
@@ -1617,4 +1608,5 @@ ROM_END
 
 GAME( 1994, rotr,    0,    rastersp, rotr,    rastersp_state, empty_init, ROT0, "BFM/Mirage", "Rise of the Robots (prototype)",        MACHINE_SUPPORTS_SAVE )
 GAME( 1994, rotra,   rotr, rastersp, rotr,    rastersp_state, empty_init, ROT0, "BFM/Mirage", "Rise of the Robots (prototype, older)", MACHINE_SUPPORTS_SAVE )
+
 GAME( 1997, fbcrazy, 0,    fbcrazy,  fbcrazy, fbcrazy_state,  empty_init, ROT0, "BFM",        "Football Crazy (Video Quiz)",           MACHINE_SUPPORTS_SAVE )
