@@ -677,20 +677,20 @@ uint8_t kc85_4_state::kc85_4_86_r()
 
 /* callback for ardy output from PIO */
 /* used in KC85/4 & KC85/3 cassette interface */
-WRITE_LINE_MEMBER( kc_state::pio_ardy_cb)
+void kc_state::pio_ardy_cb(int state)
 {
 	m_ardy = state & 0x01;
 }
 
 /* callback for brdy output from PIO */
 /* used in KC85/4 & KC85/3 keyboard interface */
-WRITE_LINE_MEMBER( kc_state::pio_brdy_cb)
+void kc_state::pio_brdy_cb(int state)
 {
 	m_brdy = state & 0x01;
 }
 
 /* used in cassette write -> K0 */
-WRITE_LINE_MEMBER( kc_state::ctc_zc0_callback )
+void kc_state::ctc_zc0_callback(int state)
 {
 	if (state)
 	{
@@ -700,7 +700,7 @@ WRITE_LINE_MEMBER( kc_state::ctc_zc0_callback )
 	}
 }
 
-WRITE_LINE_MEMBER( kc85_3_state::ctc_zc0_callback )
+void kc85_3_state::ctc_zc0_callback(int state)
 {
 	if (state)
 	{
@@ -712,7 +712,7 @@ WRITE_LINE_MEMBER( kc85_3_state::ctc_zc0_callback )
 }
 
 /* used in cassette write -> K1 */
-WRITE_LINE_MEMBER( kc_state::ctc_zc1_callback)
+void kc_state::ctc_zc1_callback(int state)
 {
 	if (state)
 	{
@@ -772,7 +772,7 @@ void kc_state::tapeout_update()
 }
 
 /* keyboard callback */
-WRITE_LINE_MEMBER( kc_state::keyboard_cb )
+void kc_state::keyboard_cb(int state)
 {
 	m_z80pio->strobe_b(state & m_brdy);
 
@@ -830,31 +830,31 @@ void kc_state::kc85_palette(palette_device &palette) const
 	{
 		// foreground colours, "full" of each component
 		{ 0x00, 0x00, 0x00 },   // black
-		{ 0x00, 0x00, 0xd0 },   // blue
-		{ 0xd0, 0x00, 0x00 },   // red
-		{ 0xd0, 0x00, 0xd0 },   // magenta
-		{ 0x00, 0xd0, 0x00 },   // green
-		{ 0x00, 0xd0, 0xd0 },   // cyan
-		{ 0xd0, 0xd0, 0x00 },   // yellow
-		{ 0xd0, 0xd0, 0xd0 },   // white
+		{ 0x00, 0x00, 0xff },   // blue
+		{ 0xff, 0x00, 0x00 },   // red
+		{ 0xff, 0x00, 0xff },   // purple
+		{ 0x00, 0xff, 0x00 },   // green
+		{ 0x00, 0xff, 0xff },   // turquoise
+		{ 0xff, 0xff, 0x00 },   // yellow
+		{ 0xff, 0xff, 0xff },   // white
 
 		// full of each component + half of another component
 		{ 0x00, 0x00, 0x00 },   // black
-		{ 0x60, 0x00, 0xa0 },   // violet
-		{ 0xa0, 0x60, 0x00 },   // brown
-		{ 0xa0, 0x00, 0x60 },   // red/purple
-		{ 0x00, 0xa0, 0x60 },   // pastel green
-		{ 0x00, 0x60, 0xa0 },   // sky blue
-		{ 0xa0, 0xa0, 0x60 },   // yellow/green
-		{ 0xd0, 0xd0, 0xd0 },   // white
+		{ 0xa0, 0x00, 0xff },   // violet
+		{ 0xff, 0xa0, 0x00 },   // orange
+		{ 0xff, 0x00, 0xa0 },   // purple-red
+		{ 0x00, 0xff, 0xa0 },   // green-blue
+		{ 0x00, 0xa0, 0xff },   // blue-green
+		{ 0xa0, 0xff, 0x00 },   // yellow-green
+		{ 0xff, 0xff, 0xff },   // white
 
 		// background colours are slightly darker than foreground colours
 		{ 0x00, 0x00, 0x00 },   // black
 		{ 0x00, 0x00, 0xa0 },   // dark blue
 		{ 0xa0, 0x00, 0x00 },   // dark red
-		{ 0xa0, 0x00, 0xa0 },   // dark magenta
+		{ 0xa0, 0x00, 0xa0 },   // dark purple
 		{ 0x00, 0xa0, 0x00 },   // dark green
-		{ 0x00, 0xa0, 0xa0 },   // dark cyan
+		{ 0x00, 0xa0, 0xa0 },   // dark turquoise
 		{ 0xa0, 0xa0, 0x00 },   // dark yellow
 		{ 0xa0, 0xa0, 0xa0 }    // dark white (grey)
 	};
@@ -863,7 +863,7 @@ void kc_state::kc85_palette(palette_device &palette) const
 }
 
 /* set new blink state */
-WRITE_LINE_MEMBER( kc_state::video_toggle_blink_state )
+void kc_state::video_toggle_blink_state(int state)
 {
 	if (state)
 	{
