@@ -32,9 +32,9 @@
 class invaders_clone_state : public invaders_state
 {
 public:
-	DECLARE_CUSTOM_INPUT_MEMBER(sicv_in2_control_r);
-	DECLARE_CUSTOM_INPUT_MEMBER(invadpt2_in1_control_r);
-	DECLARE_CUSTOM_INPUT_MEMBER(invadpt2_in2_control_r);
+	ioport_value sicv_in2_control_r();
+	ioport_value invadpt2_in1_control_r();
+	ioport_value invadpt2_in2_control_r();
 
 protected:
 	invaders_clone_state(const machine_config &mconfig, device_type type, const char *tag) :
@@ -102,7 +102,6 @@ public:
 	void lupin3a(machine_config &config);
 	void indianbt(machine_config &config);
 	void starw1(machine_config &config);
-	void cosmo(machine_config &config);
 	void spcewars(machine_config &config);
 	void cosmicmo(machine_config &config);
 	void ballbomb(machine_config &config);
@@ -115,8 +114,8 @@ public:
 
 	void init_attackfc();
 
-	DECLARE_READ_LINE_MEMBER(cosmicmo_cab_r);
-	DECLARE_READ_LINE_MEMBER(sflush_80_r);
+	int cosmicmo_cab_r();
+	int sflush_80_r();
 
 protected:
 	virtual void video_start() override { m_color_map = m_screen_red = 0; }
@@ -164,7 +163,6 @@ private:
 	void spcewars_sh_port_w(uint8_t data);
 	void lrescue_sh_port_1_w(uint8_t data);
 	void lrescue_sh_port_2_w(uint8_t data);
-	void cosmo_sh_port_2_w(uint8_t data);
 	void ballbomb_01_w(uint8_t data);
 	void ballbomb_sh_port_1_w(uint8_t data);
 	void ballbomb_sh_port_2_w(uint8_t data);
@@ -198,7 +196,6 @@ private:
 
 	void sflush_palette(palette_device &palette) const;
 
-	uint32_t screen_update_cosmo(screen_device &screen, bitmap_rgb32 &bitmap, const rectangle &cliprect);
 	uint32_t screen_update_schaser(screen_device &screen, bitmap_rgb32 &bitmap, const rectangle &cliprect);
 	uint32_t screen_update_schasercv(screen_device &screen, bitmap_rgb32 &bitmap, const rectangle &cliprect);
 	uint32_t screen_update_sflush(screen_device &screen, bitmap_rgb32 &bitmap, const rectangle &cliprect);
@@ -207,7 +204,7 @@ private:
 	uint32_t screen_update_polaris(screen_device &screen, bitmap_rgb32 &bitmap, const rectangle &cliprect);
 	uint32_t screen_update_ballbomb(screen_device &screen, bitmap_rgb32 &bitmap, const rectangle &cliprect);
 
-	DECLARE_WRITE_LINE_MEMBER(polaris_60hz_w);
+	void polaris_60hz_w(int state);
 	TIMER_DEVICE_CALLBACK_MEMBER(schaser_effect_555_cb);
 	void indianbt_sh_port_3_w(uint8_t data);
 	void polaris_sh_port_1_w(uint8_t data);
@@ -216,31 +213,29 @@ private:
 
 	void schaser_reinit_555_time_remain();
 
-	void astropal_io_map(address_map &map);
-	void attackfc_io_map(address_map &map);
-	void attackfcu_io_map(address_map &map);
-	void ballbomb_io_map(address_map &map);
-	void cosmicmo_io_map(address_map &map);
-	void cosmo_io_map(address_map &map);
-	void cosmo_map(address_map &map);
-	void crashrd_io_map(address_map &map);
-	void escmars_map(address_map &map);
-	void indianbt_io_map(address_map &map);
-	void indianbtbr_io_map(address_map &map);
-	void invadpt2_io_map(address_map &map);
-	void lrescue_io_map(address_map &map);
-	void lrescuem2_io_map(address_map &map);
-	void lupin3_io_map(address_map &map);
-	void polaris_io_map(address_map &map);
-	void schaser_io_map(address_map &map);
-	void schaser_map(address_map &map);
-	void schasercv_io_map(address_map &map);
-	void sflush_map(address_map &map);
-	void spacerng_io_map(address_map &map);
-	void spcewarla_io_map(address_map &map);
-	void spcewars_io_map(address_map &map);
-	void starw1_io_map(address_map &map);
-	void steelwkr_io_map(address_map &map);
+	void astropal_io_map(address_map &map) ATTR_COLD;
+	void attackfc_io_map(address_map &map) ATTR_COLD;
+	void attackfcu_io_map(address_map &map) ATTR_COLD;
+	void ballbomb_io_map(address_map &map) ATTR_COLD;
+	void cosmicmo_io_map(address_map &map) ATTR_COLD;
+	void crashrd_io_map(address_map &map) ATTR_COLD;
+	void escmars_map(address_map &map) ATTR_COLD;
+	void indianbt_io_map(address_map &map) ATTR_COLD;
+	void indianbtbr_io_map(address_map &map) ATTR_COLD;
+	void invadpt2_io_map(address_map &map) ATTR_COLD;
+	void lrescue_io_map(address_map &map) ATTR_COLD;
+	void lrescuem2_io_map(address_map &map) ATTR_COLD;
+	void lupin3_io_map(address_map &map) ATTR_COLD;
+	void polaris_io_map(address_map &map) ATTR_COLD;
+	void schaser_io_map(address_map &map) ATTR_COLD;
+	void schaser_map(address_map &map) ATTR_COLD;
+	void schasercv_io_map(address_map &map) ATTR_COLD;
+	void sflush_map(address_map &map) ATTR_COLD;
+	void spacerng_io_map(address_map &map) ATTR_COLD;
+	void spcewarla_io_map(address_map &map) ATTR_COLD;
+	void spcewars_io_map(address_map &map) ATTR_COLD;
+	void starw1_io_map(address_map &map) ATTR_COLD;
+	void steelwkr_io_map(address_map &map) ATTR_COLD;
 };
 
 
@@ -269,7 +264,7 @@ public:
 	void invasion(machine_config &config);
 
 private:
-	void io_map(address_map &map);
+	void io_map(address_map &map) ATTR_COLD;
 };
 
 
@@ -288,16 +283,16 @@ public:
 	void darthvdr(machine_config &config);
 
 protected:
-	virtual void machine_start() override;
-	virtual void machine_reset() override;
+	virtual void machine_start() override ATTR_COLD;
+	virtual void machine_reset() override ATTR_COLD;
 
 private:
 	void darthvdr_00_w(uint8_t data);
 	void darthvdr_08_w(uint8_t data);
 	IRQ_CALLBACK_MEMBER(darthvdr_interrupt_vector);
 
-	void main_map(address_map &map);
-	void io_map(address_map &map);
+	void main_map(address_map &map) ATTR_COLD;
+	void io_map(address_map &map) ATTR_COLD;
 
 	uint8_t m_port_1_last = 0;
 	uint8_t m_fleet_step = 0;
@@ -324,8 +319,8 @@ public:
 private:
 	uint32_t screen_update_spacecom(screen_device &screen, bitmap_rgb32 &bitmap, const rectangle &cliprect);
 
-	void main_map(address_map &map);
-	void io_map(address_map &map);
+	void main_map(address_map &map) ATTR_COLD;
+	void io_map(address_map &map) ATTR_COLD;
 
 	required_device<palette_device> m_palette;
 };
@@ -349,7 +344,7 @@ public:
 	void init_invrvnge();
 
 protected:
-	virtual void machine_start() override;
+	virtual void machine_start() override ATTR_COLD;
 
 private:
 	void port03_w(uint8_t data);
@@ -357,8 +352,8 @@ private:
 
 	TIMER_DEVICE_CALLBACK_MEMBER(nmi_timer);
 
-	void io_map(address_map &map);
-	void sound_map(address_map &map);
+	void io_map(address_map &map) ATTR_COLD;
+	void sound_map(address_map &map) ATTR_COLD;
 
 	required_device<cpu_device> m_audiocpu;
 
@@ -386,7 +381,7 @@ public:
 private:
 	uint32_t screen_update_vortex(screen_device &screen, bitmap_rgb32 &bitmap, const rectangle &cliprect);
 
-	void io_map(address_map &map);
+	void io_map(address_map &map) ATTR_COLD;
 };
 
 
@@ -403,14 +398,16 @@ public:
 	}
 
 	void rollingc(machine_config &config);
+	void mraker(machine_config &config);
 
-	DECLARE_CUSTOM_INPUT_MEMBER(game_select_r);
+	ioport_value game_select_r();
 
 protected:
-	virtual void machine_start() override;
+	virtual void machine_start() override ATTR_COLD;
 
 private:
 	void rollingc_sh_port_w(uint8_t data);
+	void mraker_sh_port1_w(uint8_t data);
 
 	uint8_t scattered_colorram_r(offs_t offset);
 	void scattered_colorram_w(offs_t offset, uint8_t data);
@@ -421,8 +418,9 @@ private:
 
 	uint32_t screen_update_rollingc(screen_device &screen, bitmap_rgb32 &bitmap, const rectangle &cliprect);
 
-	void main_map(address_map &map);
-	void io_map(address_map &map);
+	void main_map(address_map &map) ATTR_COLD;
+	void rollingc_io_map(address_map &map) ATTR_COLD;
+	void mraker_io_map(address_map &map) ATTR_COLD;
 
 	std::unique_ptr<uint8_t []> m_scattered_colorram;
 	std::unique_ptr<uint8_t []> m_scattered_colorram2;
@@ -437,9 +435,10 @@ private:
 class ozmawars_state : public _8080bw_state
 {
 public:
-	ozmawars_state(const machine_config &mconfig, device_type type, const char *tag)
-		: _8080bw_state(mconfig, type, tag)
-		{ }
+	ozmawars_state(const machine_config &mconfig, device_type type, const char *tag) :
+		_8080bw_state(mconfig, type, tag)
+	{
+	}
 
 	void ozmawars(machine_config &config);
 	void ozmawars_samples_audio(machine_config &config);
@@ -448,12 +447,54 @@ private:
 	void ozmawars_port03_w(uint8_t data);
 	void ozmawars_port04_w(uint8_t data);
 	void ozmawars_port05_w(uint8_t data);
-	void ozmawars_io_map(address_map &map);
-	u8 m_port03 = 0;
-	u8 m_port05 = 0;
+	void ozmawars_io_map(address_map &map) ATTR_COLD;
+
+	uint8_t m_port03 = 0;
+	uint8_t m_port05 = 0;
 	bool m_sound_enable = 0;
 };
 
+/*************************************/
+/* Cosmo                     */
+/*************************************/
+
+class cosmo_state : public _8080bw_state
+{
+public:
+	cosmo_state(const machine_config &mconfig, device_type type, const char *tag) :
+		_8080bw_state(mconfig, type, tag)
+	{
+	}
+
+	void cosmo(machine_config &config) ATTR_COLD;
+
+protected:
+	virtual void machine_start() override ATTR_COLD;
+
+private:
+	static constexpr uint32_t STAR_RNG_PERIOD = ((1 << 17) - 1);
+	uint32_t m_rng_offs = 0;
+	uint8_t m_stars_sidescroll = 0;
+	uint8_t m_star_speed = 3;
+	uint8_t m_bright_star = 0;
+	uint8_t m_stars[STAR_RNG_PERIOD] {};
+	uint32_t m_star_rng_origin = 0;
+	uint32_t m_star_rng_origin_frame = 0;
+
+	void sh_port_2_w(uint8_t data);
+	uint8_t stars_r();
+	void stars_w(uint8_t data);
+	void palette_init(palette_device &palette) const ATTR_COLD;
+	void stars_init() ATTR_COLD;
+	void stars_update_origin();
+	void set_pixel(bitmap_rgb32 &bitmap, uint8_t y, uint32_t x, int color);
+	void set_8_pixels(bitmap_rgb32 &bitmap, uint8_t y, uint32_t x, uint8_t data, int fore_color, int back_color);
+	void stars_draw_row(bitmap_rgb32 &bitmap, int maxx, int y, uint32_t star_offs, uint8_t starmask);
+	uint32_t screen_update(screen_device &screen, bitmap_rgb32 &bitmap, const rectangle &cliprect);
+	void io_map(address_map &map) ATTR_COLD;
+	void program_map(address_map &map) ATTR_COLD;
+};
+
 
 /*******************************************************/
 /* Wing Yosaku to Donbei                               */
@@ -470,17 +511,17 @@ public:
 	void yosakdon(machine_config &config);
 
 protected:
-	virtual void machine_start() override;
+	virtual void machine_start() override ATTR_COLD;
 
 private:
 	void sh_port_1_w(uint8_t data);
 	void sh_port_2_w(uint8_t data);
 
-	void main_map(address_map &map);
-	void io_map(address_map &map);
+	void main_map(address_map &map) ATTR_COLD;
+	void io_map(address_map &map) ATTR_COLD;
 
-	uint8_t m_port_1_last;
-	uint8_t m_port_2_last;
+	uint8_t m_port_1_last = 0;
+	uint8_t m_port_2_last = 0;
 };
 
 
@@ -502,7 +543,7 @@ public:
 	void shuttlei(machine_config &config);
 
 protected:
-	virtual void machine_start() override;
+	virtual void machine_start() override ATTR_COLD;
 
 private:
 	uint8_t port_ff_r();
@@ -512,8 +553,8 @@ private:
 
 	uint32_t screen_update_shuttlei(screen_device &screen, bitmap_rgb32 &bitmap, const rectangle &cliprect);
 
-	void main_map(address_map &map);
-	void io_map(address_map &map);
+	void main_map(address_map &map) ATTR_COLD;
+	void io_map(address_map &map) ATTR_COLD;
 
 	required_ioport m_inputs;
 	required_ioport m_p2;
@@ -542,17 +583,17 @@ public:
 
 	DECLARE_INPUT_CHANGED_MEMBER(gun_trigger);
 
-	DECLARE_READ_LINE_MEMBER(gun_on_r);
+	int gun_on_r();
 
 protected:
-	virtual void machine_start() override;
+	virtual void machine_start() override ATTR_COLD;
 
 private:
 	uint8_t gun_lo_r();
 	uint8_t gun_hi_r();
 	TIMER_DEVICE_CALLBACK_MEMBER(gun_callback);
 
-	void io_map(address_map &map);
+	void io_map(address_map &map) ATTR_COLD;
 
 	required_ioport m_gunx;
 	required_ioport m_guny;
@@ -578,11 +619,11 @@ public:
 	void cane_audio(machine_config &config);
 
 protected:
-	void cane_unknown_port0_w(u8 data);
+	void cane_unknown_port0_w(uint8_t data);
 
 private:
-	void cane_io_map(address_map &map);
-	void cane_map(address_map &map);
+	void cane_io_map(address_map &map) ATTR_COLD;
+	void cane_map(address_map &map) ATTR_COLD;
 };
 
 DISCRETE_SOUND_EXTERN( cane_discrete );
@@ -604,14 +645,14 @@ public:
 	void orbite(machine_config &config);
 
 protected:
-	virtual void machine_start() override;
+	virtual void machine_start() override ATTR_COLD;
 
-	u8 orbite_scattered_colorram_r(address_space &space, offs_t offset, u8 mem_mask = 0xff);
-	void orbite_scattered_colorram_w(address_space &space, offs_t offset, u8 data, u8 mem_mask = 0xff);
+	uint8_t orbite_scattered_colorram_r(address_space &space, offs_t offset, uint8_t mem_mask = 0xff);
+	void orbite_scattered_colorram_w(address_space &space, offs_t offset, uint8_t data, uint8_t mem_mask = 0xff);
 
 private:
-	void orbite_io_map(address_map &map);
-	void orbite_map(address_map &map);
+	void orbite_io_map(address_map &map) ATTR_COLD;
+	void orbite_map(address_map &map) ATTR_COLD;
 
 	u32 screen_update_orbite(screen_device &screen, bitmap_rgb32 &bitmap, const rectangle &cliprect);
 
@@ -639,14 +680,14 @@ public:
 	void init_invmulti();
 
 protected:
-	virtual void machine_start() override;
+	virtual void machine_start() override ATTR_COLD;
 
 private:
 	uint8_t eeprom_r();
 	void eeprom_w(uint8_t data);
 	void bank_w(uint8_t data);
 
-	void main_map(address_map &map);
+	void main_map(address_map &map) ATTR_COLD;
 
 	required_memory_bank_array<2> m_banks;
 	required_device<eeprom_serial_93cxx_device> m_eeprom;
