@@ -1,5 +1,6 @@
 // license:BSD-3-Clause
 // copyright-holders:Andrei I. Holub
+// thanks-to:Andrew Owen
 /**********************************************************************
     Chloe 280SE
 **********************************************************************/
@@ -50,7 +51,7 @@ public:
 		, m_bank1_view(*this, "bank1_view")
 		, m_regs_map(*this, "regs_map")
 		, m_palette(*this, "palette")
-		, m_ula(*this, "ula")
+		, m_ula_scr(*this, "ula_scr")
 		, m_sdcard(*this, "sdcard")
 		, m_io_line(*this, "IO_LINE%u", 0U)
 		, m_io_mouse(*this, "mouse_input%u", 1U)
@@ -103,10 +104,10 @@ private:
 	memory_view m_bank0_view, m_bank1_view;
 	required_device<address_map_bank_device> m_regs_map;
 	required_device<device_palette_interface> m_palette;
-	required_device<screen_ula_plus_device> m_ula;
+	required_device<screen_ula_plus_device> m_ula_scr;
 	required_device<spi_sdcard_device> m_sdcard;
 	required_ioport_array<8> m_io_line;
-	required_ioport_array<3> m_io_mouse;
+	required_ioport_array<4> m_io_mouse;
 	required_device<dac_byte_interface> m_covox;
 	required_device<kbdc8042_device> m_kbdc;
 
@@ -139,7 +140,7 @@ private:
 void chloe_state::update_memory()
 {
 	m_screen->update_now();
-	m_ula->ula_shadow_en_w(BIT(m_port_7ffd_data, 3));
+	m_ula_scr->ula_shadow_en_w(BIT(m_port_7ffd_data, 3));
 
 	const bool ext = BIT(m_port_ff_data, 7); // 0 - DOC 7xxxx=28+; 1 - EXT 6xxxx=24+
 	m_bank0_view.disable();
@@ -225,10 +226,10 @@ u32 chloe_state::screen_update(screen_device &screen, bitmap_rgb32 &bitmap, cons
 	clip256x192 &= cliprect;
 
 	screen.priority().fill(0, cliprect);
-	m_ula->draw_border(bitmap, cliprect, m_port_fe_data & 0x07);
+	m_ula_scr->draw_border(bitmap, cliprect, m_port_fe_data & 0x07);
 
 	const bool flash = u64(screen.frame_number() / m_frame_invert_count) & 1;
-	m_ula->draw(screen, bitmap, clip256x192, flash, 0);
+	m_ula_scr->draw(screen, bitmap, clip256x192, flash, 0);
 
 	return 0;
 }
@@ -246,7 +247,7 @@ void chloe_state::port_7ffd_w(u8 data)
 
 void chloe_state::port_ff_w(u8 data)
 {
-	m_ula->port_ff_reg_w(data);
+	m_ula_scr->port_ff_reg_w(data);
 
 	m_port_ff_data = data;
 	update_memory();
@@ -525,7 +526,7 @@ void chloe_state::map_io(address_map &map)
 			}
 			else if ((m_palpen_selected & 0xc0) == 0x40)
 			{
-				m_ula->ulap_en_w(data & 1);
+				m_ula_scr->ulap_en_w(data & 1);
 			}
 		}));
 	map(0xfc3b, 0xfc3b).lrw8(NAME([this]() { return m_reg_selected; })
@@ -538,9 +539,9 @@ void chloe_state::map_io(address_map &map)
 	map(0x00df, 0x00df).mirror(0xff00).lrw8(NAME([this]() -> u8 { return m_io_joy2->read() & 0x1f; })  // Kempston 2
 		, NAME([this](u8 data) { m_covox->data_w(data); }));
 	map(0x00b3, 0x00b3).mirror(0xff00).lw8(NAME([this](u8 data) { m_covox->data_w(data); }));
-	map(0xfadf, 0xfadf).lr8(NAME([this]() -> u8 { return 0x80 | (m_io_mouse[2]->read() & 0x07); }));
-	map(0xfbdf, 0xfbdf).lr8(NAME([this]() -> u8 { return  m_io_mouse[0]->read(); }));
-	map(0xffdf, 0xffdf).lr8(NAME([this]() -> u8 { return ~m_io_mouse[1]->read(); }));
+	map(0xfadf, 0xfadf).lr8(NAME([this]() -> u8 { return (m_io_mouse[3]->read() << 4) | m_io_mouse[2]->read(); }));
+	map(0xfbdf, 0xfbdf).lr8(NAME([this]() -> u8 { return m_io_mouse[0]->read(); }));
+	map(0xffdf, 0xffdf).lr8(NAME([this]() -> u8 { return m_io_mouse[1]->read(); }));
 
 	map(0x00f7, 0x00f7).mirror(0xff00).nopw(); // Audio Mixer. No support for now, using default ACB
 	map(0x8e3b, 0x8e3b).nopw(); // PRISMSPEEDCTRL used by software compatible with Prism
@@ -797,12 +798,16 @@ INPUT_PORTS_START(chloe)
 	PORT_BIT(0xff, 0, IPT_MOUSE_X) PORT_SENSITIVITY(30)
 
 	PORT_START("mouse_input2")
-	PORT_BIT(0xff, 0, IPT_MOUSE_Y) PORT_SENSITIVITY(30)
+	PORT_BIT(0xff, 0, IPT_MOUSE_Y) PORT_REVERSE PORT_SENSITIVITY(30)
 
 	PORT_START("mouse_input3")
-	PORT_BIT(0x01, IP_ACTIVE_LOW, IPT_BUTTON4) PORT_NAME("Left mouse button") PORT_CODE(MOUSECODE_BUTTON1)
-	PORT_BIT(0x02, IP_ACTIVE_LOW, IPT_BUTTON5) PORT_NAME("Right mouse button") PORT_CODE(MOUSECODE_BUTTON2)
-	PORT_BIT(0x04, IP_ACTIVE_LOW, IPT_BUTTON6) PORT_NAME("Middle mouse button") PORT_CODE(MOUSECODE_BUTTON3)
+	PORT_BIT(0x01, IP_ACTIVE_LOW, IPT_BUTTON4) PORT_NAME("Mouse Button Left") PORT_CODE(MOUSECODE_BUTTON1)
+	PORT_BIT(0x02, IP_ACTIVE_LOW, IPT_BUTTON5) PORT_NAME("Mouse Button Right") PORT_CODE(MOUSECODE_BUTTON2)
+	PORT_BIT(0x04, IP_ACTIVE_LOW, IPT_BUTTON6) PORT_NAME("Mouse Button Middle") PORT_CODE(MOUSECODE_BUTTON3)
+	PORT_BIT(0xf8, IP_ACTIVE_HIGH, IPT_UNUSED)
+
+	PORT_START("mouse_input4")
+	PORT_BIT(0x0f, 0, IPT_DIAL_V) PORT_REVERSE PORT_NAME("Mouse Scroll V") PORT_SENSITIVITY(1) PORT_CODE(MOUSECODE_Z)
 
 INPUT_PORTS_END
 
@@ -889,10 +894,9 @@ GFXDECODE_END
 void chloe_state::video_start()
 {
 	spectrum_128_state::video_start();
-	m_contention_pattern = {}; // Has no contention
 
 	const u8 *ram = m_ram->pointer();
-	m_ula->set_host_ram_ptr(ram);
+	m_ula_scr->set_host_ram_ptr(ram);
 }
 
 
@@ -931,21 +935,22 @@ void chloe_state::chloe(machine_config &config)
 	m_screen->set_raw(25.175_MHz_XTAL, CYCLES_HORIZ, CYCLES_VERT, SCR_FULL); // VGA
 	m_screen->set_screen_update(FUNC(chloe_state::screen_update));
 	m_screen->set_no_palette();
-
 	PALETTE(config, m_palette, FUNC(chloe_state::spectrum_palette), 256);
-	SCREEN_ULA_PLUS(config, m_ula, 0).set_raster_offset(SCR_256x192.left(), SCR_256x192.top()).set_palette(m_palette->device().tag(), 0x000, 0x000);
+	SPECTRUM_ULA_UNCONTENDED(config.replace(), m_ula);
 
-	SPEAKER(config, "speaker2", 2).front();
+	SCREEN_ULA_PLUS(config, m_ula_scr, 0).set_raster_offset(SCR_256x192.left(), SCR_256x192.top()).set_palette(m_palette->device().tag(), 0x000, 0x000);
+
+	SPEAKER(config.replace(), "speakers", 2).front();
 
 	AY_SLOT(config.replace(), "ay_slot", 28_MHz_XTAL / 16, default_ay_slot_devices, "ay_turbosound")
-		.add_route(0, "speaker2", 0.50, 0)
-		.add_route(2, "speaker2", 0.25, 0)
-		.add_route(2, "speaker2", 0.25, 1)
-		.add_route(1, "speaker2", 0.50, 1);
+		.add_route(0, "speakers", 0.50, 0)
+		.add_route(2, "speakers", 0.25, 0)
+		.add_route(2, "speakers", 0.25, 1)
+		.add_route(1, "speakers", 0.50, 1);
 
 	DAC_8BIT_R2R(config, m_covox, 0)
-		.add_route(ALL_OUTPUTS, "speaker2", 0.75, 0)
-		.add_route(ALL_OUTPUTS, "speaker2", 0.75, 1);
+		.add_route(ALL_OUTPUTS, "speakers", 0.75, 0)
+		.add_route(ALL_OUTPUTS, "speakers", 0.75, 1);
 
 	KBDC8042(config, m_kbdc);
 	m_kbdc->set_keyboard_type(kbdc8042_device::KBDC8042_STANDARD);
